package net.sf.multiproject;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.LinkedList;
//import java.util.Queue;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import javax.swing.JOptionPane;

/*
File: FileHelpers.java
License: The MIT License

Copyright (c) 2008 Manuel Haim

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/**
 * Helper methods for handling files and directories.
 * @author Manuel Haim
 * @version 2008.04.18
 */
public class FileHelpers {

	/**
	 * Open a file or directory using its standard application.
	 * This is intended to work on MS Windows, Linux (Gnome, KDE) and MacOS.
	 * @param file The file.
	 */
	public static void openDocument(File file) {
		try {
			String os = System.getProperty("os.name").toLowerCase();
			if(os.startsWith("windows")) {
				Runtime.getRuntime().exec(new String[] {
					"rundll32",
					"SHELL32.DLL,ShellExec_RunDLL",
					file.getAbsolutePath()
				});
			} else if(os.startsWith("linux") || os.startsWith("solaris") || os.startsWith("sunos")) {
				File kfmclient = new File("/usr/bin/kfmclient");
				File gnomeopen = new File("/usr/bin/gnome-open");
				if(kfmclient.exists()) {
				//if(System.getenv("KDE_FULL_SESSION").equals("true")) {
					Runtime.getRuntime().exec(new String[] {
						"/usr/bin/kfmclient",
						"exec",
						file.getAbsolutePath()
					});
				//} else if(!System.getenv("GNOME_DESKTOP_SESSION_ID").equals("")) {
				} else if(gnomeopen.exists()) {
					Runtime.getRuntime().exec(new String[] {
						"/usr/bin/gnome-open",
						file.getAbsolutePath()
					});
				} else {
					JOptionPane.showMessageDialog(null, "Linux detected, but neither"
					+ "\n/usr/bin/gnome-open (Gnome) nor\n/usr/bin/kfmclient (KDE) found.");
				}
				
			} else if(os.startsWith("mac")) {
				Runtime.getRuntime().exec(new String[] {
					"open",
					file.getAbsolutePath()
				});
				
			} else {
				JOptionPane.showMessageDialog(null, "No supported document launcher detected.");
			}
			
			// Java 1.6: Desktop.getDesktop.open("/path/to/file.pdf");
		} catch (IOException e) {e.printStackTrace();}
	}

	/**
	 * Unpack zip file.
	 * @param file2 The zip file.
	 * @param destDir The destination directory (will be created if not already existing).
	 * @param overwrite true to overwrite existing files, false otherwise.
	 */
	public static void unpack(File file2, File destDir, boolean overwrite) {
		ZipFile zipFile = null;
		
		// no overwrite
		//if(destDir.exists()) return;
		
		System.out.println("Unpack " + file2.getAbsolutePath() + "\n to " + destDir.getAbsolutePath());
		destDir.mkdirs();
		try {
			zipFile = new ZipFile(file2);
		
	        String destDirPath = addFileSeparator(destDir.getAbsolutePath());
	        for(Enumeration e = zipFile.entries(); e.hasMoreElements();) {
	            ZipEntry ze = (ZipEntry) e.nextElement();
	            System.out.println(ze.getName());
	            File file = new File(destDirPath + ze.getName());
	            if(ze.isDirectory()) {
	            	// make empty directory
	            	file.mkdirs();
	            	if(!file.isDirectory()) continue;
	            	file.setLastModified(ze.getTime());
	            } else {
	            	// extract file
	        		InputStream is = null;
	        		OutputStream os = null;
	            	try {
	            		if(file.isFile()) {
	            			if(overwrite) {
	            				if(!file.delete()) continue;
	            			} else {
	            				continue;
	            			}
	            		}
			            is = new BufferedInputStream(zipFile.getInputStream(ze));
			            File dir = new File(file.getParent());
			            dir.mkdirs();
			            os = new BufferedOutputStream(new FileOutputStream(file));
			            for(int c; (c=is.read()) != -1;) {
			                os.write((byte) c);
			            }
	            	} catch(IOException e1) {
	        			e1.printStackTrace();
	        		} finally {
	        			try { if(is!=null) is.close(); } catch (IOException e1) {}
	        			try { if(os!=null) os.close(); } catch (IOException e1) {}
	        		}
	        		if(file.exists()) file.setLastModified(ze.getTime());
	            }
	        }
		} catch(IOException e) {
			e.printStackTrace();
		} finally {
			try { if(zipFile!=null) zipFile.close(); } catch (IOException e) {}
		}
		
	}

	/**
	 * Pack a directory to a zip/jar file. If destination file exists,
	 * it will not be overwritten.
	 * @param dir Path to the directory.
	 * @param destFile Path to the destination file.
	 * @param nojavadoc true to exclude 'doc' dir, false otherwise.
	 */
	public static void pack(File dir, File destFile, boolean nojavadoc) {
		if(dir.listFiles().length == 0) return;
		if(destFile.exists()) return;
		
		String srcDir = addFileSeparator(dir.getAbsolutePath());
		System.out.println("Pack " + srcDir + "\n to " + destFile.getAbsolutePath());
		System.out.println(" nojavadoc:" + nojavadoc);
	
		try {
			ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(destFile));
			
			ZipEntry ze;
			
			LinkedList queue = new LinkedList();
			queue.add(dir);
			
			while(queue.size()!=0) {
				File current = (File) queue.removeFirst();
				if(current.isDirectory() && !current.getName().equals(".") && !current.getName().equals("..")) {
					String name = addFileSeparator(current.getAbsolutePath());
					if(name.startsWith(srcDir)) name = name.substring(srcDir.length());
					else continue;
					if(nojavadoc && name.equals(addFileSeparator("doc"))) continue;
					
					File[] currentFiles = current.listFiles();
					for (int i = 0; i < currentFiles.length; i++) {
						queue.add(currentFiles[i]);
					}
					
					if(addFileSeparator(name).replace(File.separatorChar,'/').equals("/")) continue;
					
					// add empty directory:
					ze = new ZipEntry(addFileSeparator(name).replace(File.separatorChar, '/'));
					System.out.println(addFileSeparator(name).replace(File.separatorChar, '/'));
					ze.setTime(current.lastModified());
					
					zos.putNextEntry(ze);
					zos.closeEntry();
				} else if(current.isFile()) {
					String name = current.getAbsolutePath();
					if(name.startsWith(srcDir)) name = name.substring(srcDir.length());
					else continue; // name = current.getName(); // should not happen
					
					ze = new ZipEntry(name.replace(File.separatorChar, '/'));
					System.out.println(name.replace(File.separatorChar, '/'));
					ze.setTime(current.lastModified());
					
					zos.putNextEntry(ze);
					BufferedInputStream bis = new BufferedInputStream(new FileInputStream(current));
					byte[] buffer = new byte[1024];
					int count;
					
					while( (count = bis.read(buffer)) != -1 ) {
						zos.write(buffer, 0, count);
						System.out.print("" + count + " ");
					}
					System.out.println();
					
					zos.closeEntry();
				}
			}
			
			zos.close();
		} catch(Exception e) {e.printStackTrace();}
	}

	/**
	 * Delete a file or a directory recursively.
	 * @param dir The file or dir to delete.
	 * @return true if deleted, false otherwise.
	 */
	public static boolean deleteDir(File dir) {
		if(dir.isDirectory()) {
			File[] files = dir.listFiles();
			for(int i=0; i<files.length; i++) {
				deleteDir(files[i]);
			}
		}
		return dir.delete();
	}

	/**
	 * Copy a file (not directory). If the path of the destination file
	 * does not exist yet, it will be created.
	 * @param src Source file.
	 * @param dest Destination file (not directory).
	 * @param overwrite true to overwrite existing files, false otherwise.
	 * @return true if successful, false otherwise.
	 */
	public static boolean copyFile(File src, File dest, boolean overwrite) {
		System.out.println("Copy " + src + "\n to " + dest);
		if(dest.exists()) {
			if(overwrite) {
				if(!dest.delete()) return false;
			} else return false;
		}
		
		File destDir = new File(dest.getParent());
		destDir.mkdirs();
		if(!destDir.isDirectory()) return false;
		
		boolean result = true;
		InputStream in = null;
		OutputStream out = null;
		try {
			in = new BufferedInputStream(new FileInputStream(src));
			out = new BufferedOutputStream(new FileOutputStream(dest));
			
			byte[] buffer = new byte[1024];
			int bytesRead;
			while ((bytesRead = in.read(buffer)) != -1) {
				out.write(buffer, 0, bytesRead);
			}
		} catch (IOException e) {
			result = false;
		} finally {
			try { if(in!=null) in.close(); } catch (IOException e) {}
			try { if(out!=null) out.close(); } catch (IOException e) {}
			dest.setLastModified(src.lastModified());
		}
		return result;
	}
	
	/**
	 * Copy a directory recursively.
	 * @param src Source directory.
	 * @param dest Destination directory (will be created if not already existing).
	 * @param overwrite true to overwrite existing files, false otherwise.
	 * @return true if successful, false otherwise.
	 */
	public static boolean copyDirectory(File src, File dest, boolean overwrite) {
		System.out.println("Copydir " + src.getAbsolutePath() + "\n to " + dest.getAbsolutePath());
		dest.mkdirs();
		if(!dest.isDirectory()) return false;
		
		File[] files = src.listFiles();
		for(int i=0; i<files.length; i++) {
			if(files[i].isFile()) {
				if(
					!copyFile(files[i],
						new File(
							FileHelpers.addFileSeparator(dest.getAbsolutePath()) + files[i].getName()
						),
						overwrite
					)
				) return false;
			}
			if(files[i].isDirectory()) {
				if(
					!copyDirectory(files[i],
						new File(
							FileHelpers.addFileSeparator(dest.getAbsolutePath()) + files[i].getName()
						),
						overwrite
					)
				) return false;
			}
		}
		return true;
	}

	/**
	 * Adds the file separator char to the path (if missing).
	 * @param path A dir path.
	 * @return The dir path with file separator.
	 */
	public static String addFileSeparator(String path) {
		if(!path.endsWith(File.separator))
			path += File.separator;
		return path;
	}

	/**
	 * Create bluej_pkg files for all subdirs but &quot;doc&quot; and &quot;META-INF&quot;.
	 * @param file The project dir.
	 */
	public static boolean createBluejpkg(File file) {
		return createBluejpkg(file.getAbsoluteFile(), true);
	}
	
	/**
	 * Recursive helper method for {@link #createBluejpkg(File)}.
	 * @param file A directory.
	 * @param root true if project root directory.
	 * @return true if bluej_pkg files have been added, false otherwise.
	 */
	private static boolean createBluejpkg(File file, boolean root) {
		if(file.isDirectory()) {
			File bluejpkg = new File (file.getAbsolutePath() + File.separator + "bluej.pkg");
			if(bluejpkg.exists() && root) return false; // assume subdirs have bluej_pkg as well.
			
			if(!bluejpkg.exists()) {
				try {
					bluejpkg.createNewFile();
				} catch (IOException e) {}
			}
			
			File[] files = file.listFiles();
			for(int i=0; i<files.length; i++) {
				if(files[i].getName().equals(".")) continue;
				if(files[i].getName().equals("..")) continue;
				if(files[i].getName().equals("doc") && root) continue;
				if(files[i].getName().equals("META-INF") && root) continue;
				createBluejpkg(files[i], false);
			}
		}
		return true;
	}

}
